@@ -30,12 +30,14 @@
 
 #include "common/gkr-buffer.h"
 #include "common/gkr-secure-memory.h"
+#include "common/gkr-unix-credentials.h"
 
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/un.h>
 
 #include <stdlib.h>
+#include <ctype.h>
 #include <stdint.h>
 #include <pthread.h>
 #include <unistd.h>
@@ -43,6 +45,7 @@
 #include <errno.h>
 #include <stdarg.h>
 #include <stdio.h>
+#include <string.h>
 
 /* TODO: module fini should call finalize_common */
 
@@ -125,7 +128,7 @@ gkr_pkcs11_warn (const char* msg, ...)
 	va_end (va);
 }
 
-#ifdef _xDEBUG 
+#ifdef _DEBUG 
 
 static void 
 gkr_pkcs11_debug (const char* msg, ...)
@@ -170,6 +173,100 @@ gkr_memory_fallback (void *p, unsigned long sz)
 	return realloc (p, sz);	
 }
 
+/* -----------------------------------------------------------------------------
+ * MODULE ARGUMENTS
+ */
+
+static void
+parse_argument (char *arg)
+{
+	char *value;
+	
+	value = arg + strcspn (arg, ":=");
+	if (!*value)
+		value = NULL;
+	else 
+		*(value++) = 0;
+
+	/* Setup the socket path from the arguments */
+	if (strcmp (arg, "socket") == 0) {
+		strncpy (socket_path, value, sizeof (socket_path));
+		socket_path[sizeof (socket_path) - 1] = 0;		
+	} else {
+		WARN(("unrecognized argument: %s", arg));
+	}
+}
+
+static void
+parse_arguments (const char *string) 
+{
+	char quote = '\0';
+	char *src, *dup, *at, *arg;
+	
+	if (!string)
+		return;
+	
+	src = dup = strdup (string);
+	if (!dup) {
+		WARN (("couldn't allocate memory for argument string"));
+		return;
+	}
+
+	arg = at = src;
+	for (src = dup; *src; src++) {
+		
+		/* Matching quote */
+		if (quote == *src) {
+			quote = '\0';
+			
+		/* Inside of quotes */
+		} else if (quote != '\0') {
+			if (*src == '\\') {
+				*at++ = *src++;
+				if (!*src) {
+					WARN (("couldn't parse argument string: %s", string));
+					goto done;
+				}
+				if (*src != quote) 
+					*at++ = '\\';
+			}
+			*at++ = *src;
+			
+		/* Space, not inside of quotes */
+		} else if (isspace(*src)) {
+			*at = 0;
+			parse_argument (arg);
+			arg = at;
+			
+		/* Other character outside of quotes */
+		} else {
+			switch (*src) {
+			case '\'':
+			case '"':
+				quote = *src;
+				break;
+			case '\\':
+				*at++ = *src++;
+				if (!*src) {
+					WARN (("couldn't parse argument string: %s", string));
+					goto done;
+				}
+				/* fall through */
+			default:
+				*at++ = *src;
+				break;
+			}
+		}
+	}
+
+	
+	if (at != arg) 
+		parse_argument (arg);
+	
+done:
+	free (dup);
+}
+
 /* -----------------------------------------------------------------------------
  * CALL SESSION
  */
@@ -255,7 +352,7 @@ call_session_destroy (CallSession* cs)
  * not ready for use after this call. 
  */
 static CallSession*
-call_session_create ()
+call_session_create (void)
 {
 	CallSession *cs = calloc (1, sizeof (CallSession));
 	if (!cs)
@@ -327,10 +424,17 @@ call_session_connect (CallSession *cs)
 		return CKR_DEVICE_ERROR;
 	}
 
+	if (gkr_unix_credentials_write (sock) < 0) {
+		close (sock);
+		WARN (("S%d: couldn't send socket credentials: %s", 
+		       cs->id, strerror (errno)));
+		return CKR_DEVICE_ERROR;
+	}
+
 	cs->socket = sock;
 	cs->call_state = CALL_READY;
-	
 	DBG (("S%d: connected", cs->id));
+	
 	return CKR_OK;
 }
 
@@ -1199,7 +1303,6 @@ gkr_C_Initialize (CK_VOID_PTR init_args)
 	CK_RV ret = CKR_OK;
 	pid_t initialize_pid;
 	char *path;
-	int l;
 	
 	DBG (("C_Initialize: enter"));
 
@@ -1217,7 +1320,6 @@ gkr_C_Initialize (CK_VOID_PTR init_args)
 
 		/* pReserved must be NULL */
 		args = init_args;
-		PREREQ (!args->pReserved, CKR_ARGUMENTS_BAD);
 
 		/* ALL supplied function pointers need to have the value either NULL or non-NULL. */
 		supplied_ok = (args->CreateMutex == NULL && args->DestroyMutex == NULL &&
@@ -1228,11 +1330,18 @@ gkr_C_Initialize (CK_VOID_PTR init_args)
 
 		/*
 		 * When the CKF_OS_LOCKING_OK flag isn't set and mutex function pointers are supplied
-		 * by an application, return an error.  DBus must be able to use its own locks.
+		 * by an application, return an error.  We must be able to use its own locks.
 		 */
 		if (!(args->flags & CKF_OS_LOCKING_OK) && (args->CreateMutex != NULL)) {
 			PREREQ (FALSE, CKR_CANT_LOCK);
 		}
+		
+		/* 
+		 * We support setting the socket path and other arguments from from the 
+		 * pReserved pointer, similar to how NSS PKCS#11 components are initialized. 
+		 */ 
+		if (args->pReserved) 
+			parse_arguments ((const char*)args->pReserved);			
 	}
 
 	/* Main initialization */
@@ -1258,16 +1367,16 @@ gkr_C_Initialize (CK_VOID_PTR init_args)
 			crypto_pid = initialize_pid;
 			
 			/* Lookup the socket path, append '.pkcs11' */
-			socket_path[0] = 0;
-			path = getenv ("GNOME_KEYRING_SOCKET");
-			if (path && path[0]) {
-				l = sizeof (socket_path) - 1;
-				strncpy (socket_path, path, l);
-				strncat (socket_path, ".pkcs11", l);
-				socket_path[l] = 0;
-				
-				DBG (("gnome-keyring pkcs11 socket is: %s", socket_path));
+			if (socket_path[0] == 0) {
+				socket_path[0] = 0;
+				path = getenv ("GNOME_KEYRING_SOCKET");
+				if (path && path[0]) {
+					snprintf (socket_path, sizeof (socket_path), "%s.pkcs11", path);
+					socket_path[sizeof (socket_path) - 1] = 0;
+				}
 			}
+			
+			DBG (("gnome-keyring pkcs11 socket is: %s", socket_path));
 		}
 
 	pthread_mutex_unlock (&global_mutex);
@@ -1596,7 +1705,7 @@ gkr_C_OpenSession (CK_SLOT_ID id, CK_FLAGS flags, CK_VOID_PTR user_data,
 		return CKR_TOKEN_NOT_PRESENT;
 
 	/* Create the session */
-	cs = call_session_create (&cs);
+	cs = call_session_create ();
 	if (cs == NULL) {
 		ret = CKR_HOST_MEMORY;
 		goto cleanup;
